#!/usr/bin/python3

from gi.repository import Gio
import os
import sys


UNITY_LAUNCHER_SETTINGS = "com.canonical.Unity.Launcher"
UNITY_LAUNCHER_FAVORITE_KEY = "favorites"
UNITY_APP_PREFIX = "application://"

GNOME_DASH_SETTINGS = "org.gnome.shell"
GNOME_DASH_FAVORITE_KEY = "favorite-apps"


def get_default_gnome_shell_favorites():
    settings = Gio.Settings.new(GNOME_DASH_SETTINGS)
    settings.delay()
    settings.reset(GNOME_DASH_FAVORITE_KEY)
    return settings.get_strv(GNOME_DASH_FAVORITE_KEY)


def migrate_unity_launchers():

    gs_settings = Gio.Settings.new(GNOME_DASH_SETTINGS)
    if not gs_settings.is_writable(GNOME_DASH_FAVORITE_KEY):
        print("Can't migrate unity keys to GNOME Shell as they are not writable")
        return
    gs_favorites = gs_settings.get_strv(GNOME_DASH_FAVORITE_KEY)

    if gs_favorites != get_default_gnome_shell_favorites():
        print("Migration prevented as GNOME Shell launchers are modified from defaults")
        return

    unity_settings = Gio.Settings.new(UNITY_LAUNCHER_SETTINGS)
    unity_favorites = unity_settings.get_strv(UNITY_LAUNCHER_FAVORITE_KEY)

    new_favorites = []
    for fav in unity_favorites:
        if not fav.startswith(UNITY_APP_PREFIX):
            continue
        # change for amazon icon
        if fav == UNITY_APP_PREFIX + "ubuntu-amazon-default.desktop":
            fav = UNITY_APP_PREFIX + "com.canonical.launcher.amazon.desktop"
        # change from u-c-c
        if fav == UNITY_APP_PREFIX + "unity-control-center.desktop":
            fav = UNITY_APP_PREFIX + "gnome-control-center.desktop"
        new_favorites.append(fav[len(UNITY_APP_PREFIX):])
    
    # we replace with unity existing keys
    gs_settings.set_strv(GNOME_DASH_FAVORITE_KEY, new_favorites)
    gs_settings.sync() # force sync to avoid race


def reset_scaling_factor():

    gs_settings = Gio.Settings.new("org.gnome.desktop.interface")
    for key in ["cursor-size", "scaling-factor", "text-scaling-factor"]:
        if not gs_settings.is_writable(key):
            print("Can't reset factor-reset key '%s': in read only." % key)
            continue
        gs_settings.reset(key)
    gs_settings.sync() # force sync to avoid race


if __name__ == "__main__":
    source = Gio.SettingsSchemaSource.get_default()
    # Unity or GS not installed
    if source.lookup(UNITY_LAUNCHER_SETTINGS, True) is None or source.lookup(GNOME_DASH_SETTINGS, True) is None:
        print("Unity or GNOME Shell not installed: no migration needed")
        sys.exit(0)

    # Only run that in the ubuntu/ubuntu-wayland main sessions
    # (we don't want to reset scale factor for people who
    # continue to run unity)
    current_session = os.getenv("DESKTOP_SESSION", "")
    if current_session not in ("ubuntu", "ubuntu-wayland"):
        sys.exit(0)

    migrate_unity_launchers()
    reset_scaling_factor()
